@@ -2,11 +2,11 @@
  * lcdtest: LCD monitor test pattern generator
  *
  * Main program
- * $Id: lcdtest.c 38 2007-06-05 23:37:28Z eric $
- * Copyright 2005, 2007 Eric Smith <eric@brouhaha.com>
+ * $Id: lcdtest.c 44 2010-01-26 06:28:14Z eric $
+ * Copyright 2005, 2007, 2010 Eric Smith <eric@brouhaha.com>
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2 as
+ * it under the terms of the GNU General Public License version 3 as
  * published by the Free Software Foundation.  Note that permission is
  * not granted to redistribute this program under the terms of any
  * other version of the General Public License.
@@ -17,8 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -32,56 +31,49 @@
 
 #include <SDL/SDL.h>
 #include <SDL/SDL_image.h>
+#include <SDL/SDL_ttf.h>
 
-#include "SFont.h"
-
-
-char *progname;
-
+#define UNUSED __attribute__ ((unused))
 
 #define QMAKESTR(x) #x
 #define MAKESTR(x) QMAKESTR(x)
 
+char *progname;
 
-char *help_teaser_message [] =
+
+const char help_teaser_message [] =
 {
-  "press / or ? for help",
-  NULL
+  "press / or ? for help"
 };
 
-char *help_message [] =
+const char help_message [] =
 {
-  "commands:",
-  "  Change foreground color:",
-  "      r:  red                w:  white",
-  "      g:  green            k:  black",
-  "      b:  blue",
-  "  Change pattern:",
-  "      s:  solid fill         h:  horizontal lines",
-  "      c:  crosshatch   v:  vertical lines",
-  "      d:  dots               x:  diagonal crosshatch",
-  "      a:  white vertical lines with two pixel pitch",
-  "  arrow keys: move lines in arrow direction",
-  "  plus:   increase spacing between lines",
-  "  minus:  decrease spacing between lines",
-  "  0..9:   select predefined line spacings",
-  "  q or Escape: exit program",
-  "  ? or /:  toggle display of help message",
-  "",
-  "lcdtest " MAKESTR(RELEASE) "    Free Software (GPLv2)",
-  "Copyright 2005-2007 Eric Smith <eric@brouhaha.com>",
-  "http://www.brouhaha.com/~eric/software/lcdtest/",
-  NULL
+  "commands:\n"
+  "  Change foreground color:\n"
+  "      r:  red              w:  white\n"
+  "      g:  green            k:  black\n"
+  "      b:  blue\n"
+  "  Change pattern:\n"
+  "      s:  solid fill       h:  horizontal lines\n"
+  "      c:  crosshatch       v:  vertical lines\n"
+  "      d:  dots             x:  diagonal crosshatch\n"
+  "      a:  white vertical lines with two pixel pitch\n"
+  "  arrow keys: move lines in arrow direction\n"
+  "  plus:   increase spacing between lines\n"
+  "  minus:  decrease spacing between lines\n"
+  "  0..9:   select predefined line spacings\n"
+  "  q or Escape: exit program\n"
+  "  ? or /:  toggle display of help message\n"
+  "\n"
+  "lcdtest " MAKESTR(RELEASE) "    Free Software (GPLv3)\n"
+  "Copyright 2005-2010 Eric Smith <eric@brouhaha.com>\n"
+  "http://www.brouhaha.com/~eric/software/lcdtest/"
 };
 
 
 void help (FILE *f)
 {
-  int i;
-
-  fprintf (f, "\n");
-  for (i = 0; help_message [i]; i++)
-    fprintf (f, "%s\n", help_message [i]);
+  fprintf (f, "%s", help_message);
 }
 
 
@@ -114,41 +106,133 @@ void fatal (int ret, char *format, ...)
 }
 
 
-extern char *help_font_xpm [];
-
-SDL_Surface *create_help_image (char **message,
-				int width, int height)
+void get_text_size (TTF_Font *font, const char *text, int *width, int *height)
 {
-  int i;
-  int y;
-  SDL_Surface *help_font_surface;
-  SDL_Surface *help_image = NULL;
-  SFont_Font *help_font;
+  *width = 0;
+  *height = 0;
 
-  help_font_surface = IMG_ReadXPMFromArray (help_font_xpm);
-
-  help_font = SFont_InitFont (help_font_surface);
-
-  help_image = SDL_CreateRGBSurface (SDL_HWSURFACE,
-				     width, 
-				     height,
-				     32,
-				     0x000000ff,
-				     0x0000ff00,
-				     0x00ff0000,
-				     0);
-  if (! help_image)
-    return NULL;
-
-  y = 0;
-
-  for (i = 0; message [i]; i++)
+  while (*text)
     {
-      SFont_Write (help_image, help_font, 0, y, message [i]);
-      y += SFont_TextHeight (help_font);
+      const char *p;
+      int n;
+      int line_width, line_height;
+      char buf [200];
+
+      p = strchr (text, '\n');
+      if (p)
+	{
+	  n = p - text;
+	  p++;
+	}
+      else
+	{
+	  n = strlen (text);
+	  p = text + n;
+	}
+
+      if (n > (sizeof (buf) - 1))
+	n = sizeof (buf) - 1;
+      if (n == 0)
+	buf [n++] = ' ';
+      else
+	strncpy (buf, text, n);
+      buf [n] = '\0';
+
+      if (TTF_SizeText (font, buf, & line_width, & line_height) != 0)
+	fatal (2, "TTF_SizeText failed (missing glyph?)\n");
+      *height += line_height;
+      if (line_width > *width)
+	*width = line_width;
+
+      text = p;
+    }
+}
+
+SDL_Surface *create_text_surface (const char *message,
+				  const char *font_path,
+				  int font_size,
+				  SDL_Color fg,
+				  SDL_Color bg UNUSED)
+{
+  TTF_Font *font;
+  int width, height;
+  SDL_Surface *text_surface;
+  SDL_Rect dest_rect;
+  int line_height;
+
+  if ((! TTF_WasInit ()) && (TTF_Init () != 0))
+    fatal (2, "can't initialize SDL_ttf\n");
+
+  font = TTF_OpenFont (font_path, font_size);
+  if (! font)
+    fatal (2, "can't open font\n");
+
+  line_height = TTF_FontLineSkip (font);
+
+  get_text_size (font, message, & width, & height);
+  printf ("width: %d, height: %d\n", width, height);
+
+  text_surface = SDL_CreateRGBSurface (SDL_HWSURFACE,
+				       width,
+				       height,
+				       32,
+				       0x000000ff,
+				       0x0000ff00,
+				       0x00ff0000,
+				       0);
+
+  dest_rect.w = width;
+  dest_rect.h = height;
+  dest_rect.x = 0;
+  dest_rect.y = 0;
+
+  while (*message)
+    {
+      const char *p;
+      int n;
+      SDL_Surface *line_surface;
+      char buf [200];
+
+      p = strchr (message, '\n');
+      if (p)
+	{
+	  n = p - message;
+	  p++;
+	}
+      else
+	{
+	  n = strlen (message);
+	  p = message + n;
+	}
+
+      if (n > (sizeof (buf) - 1))
+	n = sizeof (buf) - 1;
+      if (n == 0)
+	buf [n++] = ' ';
+      else
+	strncpy (buf, message, n);
+      buf [n] = '\0';
+
+      if (n)
+	{
+	  line_surface = TTF_RenderText_Blended (font, buf, fg);
+	  if (! line_surface)
+	    fatal (2, "can't render text\n");
+	}
+
+      SDL_BlitSurface (line_surface,
+		       & line_surface->clip_rect,
+		       text_surface,
+		       & dest_rect);
+      SDL_FreeSurface (line_surface);
+
+      dest_rect.y += line_height;
+
+      message = p;
     }
 
-  return (help_image);
+  TTF_CloseFont (font);
+  return (text_surface);
 }
 
 
@@ -279,7 +363,7 @@ void draw_diagonal_lines (SDL_Surface *screen,
 			  uint32_t color,
 			  int h_pitch,
 			  int h_offset,
-			  int v_pitch,
+			  int v_pitch UNUSED,
 			  int v_offset,
 			  int slope)  // -1 or +1 if 45 degree angle
 {
@@ -658,6 +742,8 @@ int enumerate_sdl_modes (int *width, int *height, FILE *out)
 }
 
 
+const char font_path [] = "/usr/share/fonts/liberation/LiberationMono-Regular.ttf";
+
 int main (int argc, char *argv [])
 {
   SDL_Surface *screen;
@@ -665,6 +751,8 @@ int main (int argc, char *argv [])
   uint32_t sdl_flags;
   int width = -1;
   int height = -1;
+  SDL_Color black = { 0x00, 0x00, 0x00, 0x00 };
+  SDL_Color white = { 0xff, 0xff, 0xff, 0x00 };
 
   progname = argv [0];
 
@@ -673,8 +761,16 @@ int main (int argc, char *argv [])
   atexit (SDL_Quit);
   SDL_EnableUNICODE (1);
 
-  help_teaser_image = create_help_image (help_teaser_message, 230, 30);
-  help_image = create_help_image (help_message, 600, 450);
+  help_teaser_image = create_text_surface (help_teaser_message,
+					   font_path,
+					   18,
+					   white,
+					   black);
+  help_image = create_text_surface (help_message,
+				    font_path,
+				    18,
+				    white,
+				    black);
 
   while (--argc)
     {
